/*
 *  NAME:
 *  	spica_struct_1a.c
 *
 *  PURPOSE:
 *	This file contains the functions used to read all the FITS extension 
 *	data and header of the SPICAM/SPICAV UV/IR data level 1A
 *
 * Author:
 *   Nicolas Chapron
 *     CNRS | LATMOS - Laboratoire Atmospheres, Milieux, Observations Spatiales.
 *     11 boulevard d'Alembert
 *     78280 Guyancourt
 *     France
 *     Tel. 33-(0) 1 80 28 50 57
 *     Email. nicolas.chapron@latmos.ipsl.fr
 *
 *  SVN Revision:
 *      $Rev: 2371 $
 *      $Author: chapron $
 *      $Date: 2014-11-03 10:50:53 +0100 (Lun, 03 nov 2014) $
 *
 */

#include "spica_struct_1A.h"

// ----- readInfo --------------------------------------------------------------
void 
read_Info(fitsfile *fptr, spica_1Afile *output,int *status_,int uvir){
	
	int nfound;
	int status = 0;
	long naxes[3];

	fits_read_keys_lng(fptr, "NAXIS", 1, 3, naxes, &nfound, &status);
	*status_ = *status_ + status;
	
	output->Info.naxis1 = naxes[0];
	output->Info.naxis2 = naxes[1];
	output->Info.naxis3 = naxes[2];
	
        // ------ Array Allocation ------
	// UV/IR
	output->Info.instrument = malloc(FLEN_VALUE);
	output->Info.obstype = malloc(FLEN_VALUE);
	output->Info.begins = malloc(FLEN_VALUE);
	output->Info.ends = malloc(FLEN_VALUE);
	
	// UV ONLY
	if(uvir == 0)
	{
		output->Info.File0C = malloc(FLEN_VALUE);
		output->Info.data_status = malloc(FLEN_VALUE);
		output->Info.geo_status = malloc(FLEN_VALUE);
		output->Info.flag_status = malloc(FLEN_VALUE);
		output->Info.dc_status = malloc(FLEN_VALUE);
	}
	
	// IR ONLY
	if(uvir == 1)
	{
		output->Info.status = malloc(FLEN_VALUE);
	}
	
	// ------ Read parameters ------
	// UV/IR
	fits_read_keyword(fptr,"INSTRU",output->Info.instrument,NULL,&status);
	*status_ = *status_ + status;
	fits_read_keyword(fptr,"OBSTYPE",output->Info.obstype,NULL,&status);
	*status_ = *status_ + status;
	fits_read_keyword(fptr,"BEGINS",output->Info.begins,NULL,&status);
	*status_ = *status_ + status;
	fits_read_keyword(fptr,"ENDS",output->Info.ends,NULL,&status);
	*status_ = *status_ + status;
	fits_read_key(fptr,TINT,"ORBIT",&output->Info.orbit,NULL,&status);
	*status_ = *status_ + status;
	fits_read_key(fptr,TINT,"SEQ_NB",&output->Info.seq,NULL,&status);
	*status_ = *status_ + status;	
	
	output->Info.instrument = trimchars(output->Info.instrument);
	output->Info.obstype = trimchars(output->Info.obstype);
	output->Info.begins = trimchars(output->Info.begins);
	output->Info.ends = trimchars(output->Info.ends);
	
	// UV ONLY
	if(uvir == 0)
	{
		fits_read_key(fptr,TINT,"DCProc",&output->Info.DCProc,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"ENProc",&output->Info.ENProc,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"COSProc",&output->Info.COSProc,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"SATProc",&output->Info.SATProc,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"ORBDCNU",&output->Info.orbDCNU,NULL,&status);
		*status_ = *status_ + status;
		fits_read_keyword(fptr,"DATA_0C",output->Info.File0C,NULL,&status);
		*status_ = *status_ + status;
		fits_read_keyword(fptr,"DATA_SS",output->Info.data_status,NULL,&status);
		*status_ = *status_ + status;
		fits_read_keyword(fptr,"GEO_SS",output->Info.geo_status,NULL,&status);
		*status_ = *status_ + status;
		fits_read_keyword(fptr,"FLAG_SS",output->Info.flag_status,NULL,&status);
		*status_ = *status_ + status;
		fits_read_keyword(fptr,"DC_SS",output->Info.dc_status,NULL,&status);
		*status_ = *status_ + status;
		
		output->Info.File0C = trimchars(output->Info.File0C);
		output->Info.data_status = trimchars(output->Info.data_status);
		output->Info.geo_status = trimchars(output->Info.geo_status);
		output->Info.flag_status = trimchars(output->Info.flag_status);
		output->Info.dc_status = trimchars(output->Info.dc_status);
	}
	
	// IR ONLY
	if(uvir == 1)
	{
		fits_read_keyword(fptr,"STATUS",output->Info.status,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"NB_SPECT",&output->Info.nb_spectrum,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"NB_POINT",&output->Info.nb_points,NULL,&status);
		*status_ = *status_ + status;		
		fits_read_key(fptr,TINT,"NB_CHANN",&output->Info.nb_channels,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"NB_BLOCK",&output->Info.nb_blocks,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"SIZE_BL",&output->Info.size_block,NULL,&status);
		*status_ = *status_ + status;	
		
		output->Info.status = trimchars(output->Info.status);
	}
	
	fits_report_error(stderr, *status_);
		
}
// ----- END readInfo ----------------------------------------------------------

// ----- read_Parameters -------------------------------------------------------
void 
read_Parameters(fitsfile *fptr, spica_1Afile *output,int *status_, int nb_spectrum, int nb_points, int uvir){
	
	int anynull;
	int status = 0;
	long naxis2,longnull;
	
        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Functional_Parameters",0,&status);
	
        // ------ Array Allocation ------
	fits_read_key(fptr,TLONG,"NAXIS2",&naxis2, NULL, &status);
	*status_ = *status_ + status;
	
	// UV ONLY
	if(uvir == 0)
	{
		output->Parameters.all_ti = malloc(naxis2*sizeof(int));
		output->Parameters.t_peltier = malloc(naxis2*sizeof(float));
		output->Parameters.t_ccd = malloc(naxis2*sizeof(float));
		output->Parameters.t_numboard = malloc(naxis2*sizeof(float));
		output->Parameters.t_btboard = malloc(naxis2*sizeof(float));
		output->Parameters.t_shutter = malloc(naxis2*sizeof(float));
		output->Parameters.t_servboard = malloc(naxis2*sizeof(float));
		output->Parameters.t_hvps = malloc(naxis2*sizeof(float));
		output->Parameters.t_structure = malloc(naxis2*sizeof(float));
	}
	
	// IR ONLY
	if(uvir == 1)
	{
		output->Parameters.frequency = malloc(nb_points*sizeof(int));
		output->Parameters.t_d0 = malloc(nb_spectrum*sizeof(int));
		output->Parameters.t_d1 = malloc(nb_spectrum*sizeof(int));
		output->Parameters.t_aotf = malloc(nb_spectrum*sizeof(int));
		output->Parameters.t_base = malloc(nb_spectrum*sizeof(int));
		output->Parameters.power_rf = malloc(nb_spectrum*sizeof(int));
		output->Parameters.pvs = malloc(nb_spectrum*sizeof(int));
	}
	
        // ------ Read parameters ------
        // UV/IR
	fits_read_key(fptr,TINT,"PELTIER",&output->Parameters.peltier,NULL,&status);
	*status_ = *status_ + status;
	fits_read_key(fptr,TINT,"SOIR_ON",&output->Parameters.soiron,NULL,&status);
	*status_ = *status_ + status;
		
	// UV ONLY
	if(uvir == 0)
	{
		fits_read_key(fptr,TINT,"CODEOP",&output->Parameters.codeop,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"BINNING",&output->Parameters.binning,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"HT",&output->Parameters.ht,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"TI",&output->Parameters.ti,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"X0",&output->Parameters.x0,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"Y0",&output->Parameters.y0,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"SLIT",&output->Parameters.slit,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"UVSAMPL",&output->Parameters.uvsampling,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"IR_ON",&output->Parameters.iron,NULL,&status);
		*status_ = *status_ + status;
	}
	
	// IR ONLY
	if(uvir == 1)
	{
		fits_read_key(fptr,TINT,"UVON",&output->Parameters.uvon,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"EXIT",&output->Parameters.exit,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"SOURCE",&output->Parameters.source,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TINT,"GAIN",&output->Parameters.gain,NULL,&status);
		*status_ = *status_ + status;				
		fits_read_key(fptr,TINT,"DAC",&output->Parameters.dac,NULL,&status);
		*status_ = *status_ + status;				
		fits_read_key(fptr,TINT,"DOTS",&output->Parameters.dots,NULL,&status);
		*status_ = *status_ + status;				
		fits_read_key(fptr,TINT,"NDOTS",&output->Parameters.ndots,NULL,&status);
		*status_ = *status_ + status;				
		fits_read_key(fptr,TFLOAT,"TIME",&output->Parameters.time,NULL,&status);
		*status_ = *status_ + status;				
		fits_read_key(fptr,TFLOAT,"GAINBST",&output->Parameters.gainboost,NULL,&status);
		*status_ = *status_ + status;	
		fits_read_key(fptr,TINT,"DETS",&output->Parameters.dets,NULL,&status);
		*status_ = *status_ + status;				
	}
	
        // ------ Read Table data ------
        // UV ONLY
	if(uvir == 0)
	{
		fits_read_col(fptr, TINT, 1, 1, 1, naxis2, &longnull, output->Parameters.all_ti,&anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Parameters.t_peltier,&anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Parameters.t_ccd,&anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Parameters.t_numboard, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Parameters.t_btboard, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Parameters.t_shutter, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 7, 1, 1, naxis2, &longnull, output->Parameters.t_servboard, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 8, 1, 1, naxis2, &longnull, output->Parameters.t_hvps, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 9, 1, 1, naxis2, &longnull, output->Parameters.t_structure, &anynull, &status);
		*status_ = *status_ + status;	
	}
	
	// IR ONLY
	if(uvir == 1)
	{
		fits_read_col(fptr, TFLOAT, 1, 1, 1, nb_points, &longnull, output->Parameters.frequency,&anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 2, 1, 1, nb_spectrum, &longnull, output->Parameters.t_d0,&anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 3, 1, 1, nb_spectrum, &longnull, output->Parameters.t_d1,&anynull, &status);
		*status_ = *status_ + status;		
		fits_read_col(fptr, TFLOAT, 4, 1, 1, nb_spectrum, &longnull, output->Parameters.t_aotf,&anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 5, 1, 1, nb_spectrum, &longnull, output->Parameters.t_base,&anynull, &status);
		*status_ = *status_ + status;		
		fits_read_col(fptr, TFLOAT, 6, 1, 1, nb_spectrum, &longnull, output->Parameters.power_rf,&anynull, &status);
		*status_ = *status_ + status;		
		fits_read_col(fptr, TFLOAT, 7, 1, 1, nb_spectrum, &longnull, output->Parameters.pvs,&anynull, &status);
		*status_ = *status_ + status;		
	}	
	
	fits_report_error(stderr, *status_);
	
}
// ----- END read_Parameters ---------------------------------------------------

// ----- read_Time_of_Records --------------------------------------------------
void 
read_Time_of_Records(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int anynull;
	int status = 0;
	long naxis2,longnull;
	
        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Time_of_Records",0,&status);
	
        // ------ Array Allocation ------
	fits_read_key(fptr,TLONG,"NAXIS2",&naxis2, NULL, &status);
	*status_ = *status_ + status;
	
	output->Time_of_Records.year = malloc(naxis2*sizeof(int));
	output->Time_of_Records.month = malloc(naxis2*sizeof(int));
	output->Time_of_Records.day = malloc(naxis2*sizeof(int));
	output->Time_of_Records.hour = malloc(naxis2*sizeof(int));
	output->Time_of_Records.minute = malloc(naxis2*sizeof(int));
	output->Time_of_Records.second = malloc(naxis2*sizeof(int));
	output->Time_of_Records.msecond = malloc(naxis2*sizeof(int));

        // ------ Read Table data ------
	fits_read_col(fptr, TINT, 1, 1, 1, naxis2, &longnull, output->Time_of_Records.year,&anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TINT, 2, 1, 1, naxis2, &longnull, output->Time_of_Records.month,&anynull, &status);
	*status_ = *status_ + status;	
	fits_read_col(fptr, TINT, 3, 1, 1, naxis2, &longnull, output->Time_of_Records.day,&anynull, &status);
	*status_ = *status_ + status;	
	fits_read_col(fptr, TINT, 4, 1, 1, naxis2, &longnull, output->Time_of_Records.hour,&anynull, &status);
	*status_ = *status_ + status;		
	fits_read_col(fptr, TINT, 5, 1, 1, naxis2, &longnull, output->Time_of_Records.minute,&anynull, &status);
	*status_ = *status_ + status;	
	fits_read_col(fptr, TINT, 6, 1, 1, naxis2, &longnull, output->Time_of_Records.second,&anynull, &status);
	*status_ = *status_ + status;		
	fits_read_col(fptr, TFLOAT, 7, 1, 1, naxis2, &longnull, output->Time_of_Records.msecond,&anynull, &status);
	*status_ = *status_ + status;		
	
	fits_report_error(stderr, *status_);
	
}
// ----- END read_Time_of_Records ----------------------------------------------

// ----- read_GeoInfo ----------------------------------------------------------
void 
read_GeoInfo(fitsfile *fptr, spica_1Afile *output, int *status_, int uvir){
	
	int ii = 0;
	int status = 0;
	char *slitc;
	char *irfovpos;
	char *delim = " ";
	char *token = NULL;
	
        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Record",0,&status);
	
        // ------ Array Allocation ------
        // UV/IR
	slitc = malloc(FLEN_VALUE);
	irfovpos = malloc(FLEN_VALUE);
	output->GeoInfo.target = malloc(FLEN_VALUE);	
	
	// UV ONLY
	if(uvir ==0)
	{
		output->GeoInfo.shadowcone = malloc(FLEN_VALUE);
	}
	
        // ------ Read parameters ------
        // UV/IR
	fits_read_keyword(fptr,"TARGET",output->GeoInfo.target,NULL,&status);
	*status_ = *status_ + status;
	fits_read_key(fptr,TFLOAT,"SUNLAT",&output->GeoInfo.sunlat,NULL,&status);
	*status_ = *status_ + status;
	fits_read_key(fptr,TFLOAT,"SUNLONG",&output->GeoInfo.sunlong,NULL,&status);
	*status_ = *status_ + status;
	fits_read_key(fptr,TFLOAT,"SUNDIST",&output->GeoInfo.sundist,NULL,&status);
	*status_ = *status_ + status;
	fits_read_key(fptr,TFLOAT,"SUNLS",&output->GeoInfo.sunls,NULL,&status);
	*status_ = *status_ + status;
	fits_read_key(fptr,TFLOAT,"SUNRA",&output->GeoInfo.sunra,NULL,&status);
	*status_ = *status_ + status;
	fits_read_key(fptr,TFLOAT,"SUNDEC",&output->GeoInfo.sundec,NULL,&status);
	*status_ = *status_ + status;
	fits_read_keyword(fptr,"SLIT_C",slitc,NULL,&status);
	*status_ = *status_ + status;
	
	slitc = trimchars(slitc);
	output->GeoInfo.target     = trimchars(output->GeoInfo.target);
	
	for (token = strtok(slitc, delim); token != NULL; token = strtok(NULL, delim))
	{
		char *unconverted;
		double value = strtod(token, &unconverted);
		if (!isspace(*unconverted) && *unconverted != 0)
		{
			printf("probleme conversion string->float\n");
		}
		output->GeoInfo.slitcenter[ii] = value;
		ii++;
	}
	
	// UV ONLY
	if(uvir ==0)
	{
		fits_read_keyword(fptr,"CONE",output->GeoInfo.shadowcone,NULL,&status);
		*status_ = *status_ + status;
		
		output->GeoInfo.shadowcone = trimchars(output->GeoInfo.shadowcone);
	}
	
	// IR ONLY
	if(uvir == 1)
	{
		fits_read_keyword(fptr,"IRFOVPOS",irfovpos,NULL,&status);
		*status_ = *status_ + status;
		fits_read_key(fptr,TFLOAT,"IRSHIFT",&output->GeoInfo.irshift,NULL,&status);
		*status_ = *status_ + status;
		
		irfovpos = trimchars(irfovpos);
		for (token = strtok(irfovpos, delim); token != NULL; token = strtok(NULL, delim))
		{
			char *unconverted;
			double value = strtod(token, &unconverted);
			if (!isspace(*unconverted) && *unconverted != 0)
			{
				printf("probleme conversion string->float\n");
			}
			output->GeoInfo.irfov_pos[ii] = value;
			ii++;
		}
	}
	
	fits_report_error(stderr, *status_);
	
}
// ----- END read_GeoInfo ------------------------------------------------------

// ----- read_Geo_Record -------------------------------------------------------
void 
read_Geo_Record(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;
	
        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Record",0,&status);
	
        // ------ Array Allocation ------
        // UV/IR
	fits_read_key(fptr,TLONG,"NAXIS2",&naxis2, NULL, &status);
	output->Geo.Record.number = malloc(naxis2*sizeof(int));
	output->Geo.Record.time   = malloc(naxis2*sizeof(&output->Geo.Record.time));
	
	for (ii=0; ii<naxis2; ii++) {
		output->Geo.Record.time[ii] = malloc(24*sizeof(char));
	}
	
        // ------ Read Table data ------
        // UV/IR
	fits_read_col(fptr, TINT, 2, 1, 1, naxis2, &longnull, output->Geo.Record.number, &anynull, &status);	
	*status_ = *status_ + status;
	fits_read_col(fptr, TSTRING, 1, 1, 1, naxis2, &longnull, output->Geo.Record.time, &anynull, &status);
	*status_ = *status_ + status;	
	
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_Record ---------------------------------------------------

// ----- read_Geo_Spacecraft ---------------------------------------------------
void 
read_Geo_Spacecraft(fitsfile *fptr, spica_1Afile *output,int *status_, int uvir){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;
	
        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Spacecraft",0,&status);
	
	// ------ Array Allocation ------
	// UV/IR
	output->Geo.Spacecraft.lat = malloc(naxis2*sizeof(float));
	output->Geo.Spacecraft.lon = malloc(naxis2*sizeof(float));
	output->Geo.Spacecraft.alt = malloc(naxis2*sizeof(float));
	output->Geo.Spacecraft.sza = malloc(naxis2*sizeof(float));
	
	// UV ONLY
	if(uvir == 0)
	{
		output->Geo.Spacecraft.shadowconeratio = malloc(naxis2*sizeof(float));
	}
	
        // ------ Read Table data ------
        // UV/IR
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Spacecraft.lat, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Spacecraft.lon, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Spacecraft.alt, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Spacecraft.sza, &anynull, &status);
	*status_ = *status_ + status;
	
	// UV ONLY
	if(uvir == 0)
	{
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Spacecraft.shadowconeratio, &anynull, &status);
		*status_ = *status_ + status;
	}
	
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_Spacecraft -----------------------------------------------

// ----- read_Geo_Band1 --------------------------------------------------------
void 
read_Geo_Band1(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;

        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Band1",0,&status);
	
	// ------ Array Allocation ------
	output->Geo.Band1.lat   = calloc(naxis2,sizeof(float));
	output->Geo.Band1.lon = calloc(naxis2,sizeof(float));
	output->Geo.Band1.alt   = calloc(naxis2,sizeof(float));
	output->Geo.Band1.ra    = calloc(naxis2,sizeof(float));
	output->Geo.Band1.dec   = calloc(naxis2,sizeof(float));
	
        // ------ Read Table data ------
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Band1.lat, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Band1.lon, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Band1.alt, &anynull, &status);	
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Band1.ra, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Band1.dec, &anynull, &status);
	*status_ = *status_ + status;
	
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_Band1 ----------------------------------------------------

// ----- read_Geo_Band2 --------------------------------------------------------
void 
read_Geo_Band2(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;
	
        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Band2",0,&status);

	// ------ Array Allocation ------
	output->Geo.Band2.lat   = calloc(naxis2,sizeof(float));
	output->Geo.Band2.lon = calloc(naxis2,sizeof(float));
	output->Geo.Band2.alt   = calloc(naxis2,sizeof(float));
	output->Geo.Band2.ra    = calloc(naxis2,sizeof(float));
	output->Geo.Band2.dec   = calloc(naxis2,sizeof(float));

        // ------ Read Table data ------
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Band2.lat, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Band2.lon, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Band2.alt, &anynull, &status);	
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Band2.ra, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Band2.dec, &anynull, &status);
	*status_ = *status_ + status;
	
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_Band2 ----------------------------------------------------

// ----- read_Geo_Band3 --------------------------------------------------------
void 
read_Geo_Band3(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;

        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Band3",0,&status);
	
	// ------ Array Allocation ------
	output->Geo.Band3.sza = malloc(naxis2*sizeof(float));
	output->Geo.Band3.disttoplanetnp = malloc(naxis2*sizeof(float));
	output->Geo.Band3.pixelsize = malloc(naxis2*sizeof(float));
	output->Geo.Band3.anglelossun = malloc(naxis2*sizeof(float));
	output->Geo.Band3.phaselossun = malloc(naxis2*sizeof(float));
	output->Geo.Band3.solarincidence = malloc(naxis2*sizeof(float));
	output->Geo.Band3.solarlocaltime = malloc(naxis2*sizeof(float));
	output->Geo.Band3.anglenormalobserver = malloc(naxis2*sizeof(float));
	output->Geo.Band3.sunazimuth = malloc(naxis2*sizeof(float));
	output->Geo.Band3.distlosplanetcenter = malloc(naxis2*sizeof(float));
	output->Geo.Band3.distscplanetcenter = malloc(naxis2*sizeof(float));
	output->Geo.Band3.lat = calloc(naxis2,sizeof(float));
	output->Geo.Band3.lon = calloc(naxis2,sizeof(float));
	output->Geo.Band3.alt = calloc(naxis2,sizeof(float));
	output->Geo.Band3.ra = calloc(naxis2,sizeof(float));
	output->Geo.Band3.dec = calloc(naxis2,sizeof(float));
	
        // ------ Read Table data ------
	fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Geo.Band3.sza, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 7, 1, 1, naxis2, &longnull, output->Geo.Band3.disttoplanetnp, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 8, 1, 1, naxis2, &longnull, output->Geo.Band3.pixelsize, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 9, 1, 1, naxis2, &longnull, output->Geo.Band3.anglelossun, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 10, 1, 1, naxis2, &longnull, output->Geo.Band3.phaselossun, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 11, 1, 1, naxis2, &longnull, output->Geo.Band3.solarincidence, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 12, 1, 1, naxis2, &longnull, output->Geo.Band3.solarlocaltime, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 13, 1, 1, naxis2, &longnull, output->Geo.Band3.anglenormalobserver,  &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 14, 1, 1, naxis2, &longnull, output->Geo.Band3.sunazimuth, &anynull, &status);	
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 15, 1, 1, naxis2, &longnull, output->Geo.Band3.distlosplanetcenter, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 16, 1, 1, naxis2, &longnull, output->Geo.Band3.distscplanetcenter, &anynull, &status);
	*status_ = *status_ + status;
	
	if (output->Parameters.codeop >= 100) {
		
		fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Band3.lat, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Band3.lon, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Band3.alt, &anynull, &status);	
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Band3.ra, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Band3.dec, &anynull, &status);
		*status_ = *status_ + status;
	}
	
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_Band3 ----------------------------------------------------

// ----- read_Geo_Band4 --------------------------------------------------------
void 
read_Geo_Band4(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;

	// ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Band4",0,&status);
	
	// ------ Array Allocation ------
	output->Geo.Band4.lat   = calloc(naxis2,sizeof(float));
	output->Geo.Band4.lon = calloc(naxis2,sizeof(float));
	output->Geo.Band4.alt   = calloc(naxis2,sizeof(float));
	output->Geo.Band4.ra    = calloc(naxis2,sizeof(float));
	output->Geo.Band4.dec   = calloc(naxis2,sizeof(float));
	
	// ------ Read Table data ------
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Band4.lat, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Band4.lon, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Band4.alt, &anynull, &status);	
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Band4.ra, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Band4.dec, &anynull, &status);
	*status_ = *status_ + status;
		
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_Band4 ----------------------------------------------------

// ----- read_Geo_Band5 --------------------------------------------------------
void 
read_Geo_Band5(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;

	// ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Band5",0,&status);
	
	// ------ Array Allocation ------
	output->Geo.Band5.lat   = calloc(naxis2,sizeof(float));
	output->Geo.Band5.lon = calloc(naxis2,sizeof(float));
	output->Geo.Band5.alt   = calloc(naxis2,sizeof(float));
	output->Geo.Band5.ra    = calloc(naxis2,sizeof(float));
	output->Geo.Band5.dec   = calloc(naxis2,sizeof(float));	
	
	// ------ Read Table data ------
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Band5.lat, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Band5.lon,  &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Band5.alt, &anynull, &status);	
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Band5.ra, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Band5.dec, &anynull, &status);
	*status_ = *status_ + status;
		
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_Band5 ----------------------------------------------------

// ----- read_Geo_CCDLine ------------------------------------------------------
void 
read_Geo_CCDLine(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;

	// ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_CCDLine",0,&status);
	
	// ------ Array Allocation ------
	output->Geo.CCDLine.lat   = calloc(naxis2,sizeof(float));
	output->Geo.CCDLine.lon = calloc(naxis2,sizeof(float));
	output->Geo.CCDLine.alt   = calloc(naxis2,sizeof(float));
	output->Geo.CCDLine.ra    = calloc(naxis2,sizeof(float));
	output->Geo.CCDLine.dec   = calloc(naxis2,sizeof(float));	
	
	// ------ Read Table data ------
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.CCDLine.lat, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.CCDLine.lon, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.CCDLine.alt, &anynull, &status);	
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.CCDLine.ra, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.CCDLine.dec, &anynull, &status);
	*status_ = *status_ + status;
	
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_CCDLine --------------------------------------------------

// ----- read_Geo_LOSE ---------------------------------------------------------
void 
read_Geo_LOSE(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;

	// ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_LOSE",0,&status);
	
	if (status == 0) 
	{
		
		// ------ Array Allocation ------
		output->Geo.LOSE.lat = calloc(naxis2,sizeof(float));
		output->Geo.LOSE.lon = calloc(naxis2,sizeof(float));
		output->Geo.LOSE.alt = calloc(naxis2,sizeof(float));
		output->Geo.LOSE.ra = calloc(naxis2,sizeof(float));
		output->Geo.LOSE.dec = calloc(naxis2,sizeof(float));
		output->Geo.LOSE.sza = malloc(naxis2*sizeof(float));
		output->Geo.LOSE.disttoplanetnp = malloc(naxis2*sizeof(float));
		output->Geo.LOSE.pixelsize = malloc(naxis2*sizeof(float));
		output->Geo.LOSE.anglelossun = malloc(naxis2*sizeof(float));
		output->Geo.LOSE.phaselossun = malloc(naxis2*sizeof(float));
		output->Geo.LOSE.solarincidence = malloc(naxis2*sizeof(float));
		output->Geo.LOSE.solarlocaltime = malloc(naxis2*sizeof(float));
		output->Geo.LOSE.anglenormalobserver = malloc(naxis2*sizeof(float));
		output->Geo.LOSE.sunazimuth = malloc(naxis2*sizeof(float));
		output->Geo.LOSE.distlosplanetcenter = malloc(naxis2*sizeof(float));
		output->Geo.LOSE.distscplanetcenter = malloc(naxis2*sizeof(float));
		
		// ------ Read Table data ------
		fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.LOSE.lat, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.LOSE.lon, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.LOSE.alt, &anynull, &status);	
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.LOSE.ra, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.LOSE.dec, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Geo.LOSE.sza, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 7, 1, 1, naxis2, &longnull, output->Geo.LOSE.disttoplanetnp, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 8, 1, 1, naxis2, &longnull, output->Geo.LOSE.pixelsize, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 9, 1, 1, naxis2, &longnull, output->Geo.LOSE.anglelossun, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 10, 1, 1, naxis2, &longnull, output->Geo.LOSE.phaselossun, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 11, 1, 1, naxis2, &longnull, output->Geo.LOSE.solarincidence, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 12, 1, 1, naxis2, &longnull, output->Geo.LOSE.solarlocaltime, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 13, 1, 1, naxis2, &longnull, output->Geo.LOSE.anglenormalobserver, &anynull, &status);
		*status_ = *status_ + status;
		fits_read_col(fptr, TFLOAT, 14, 1, 1, naxis2, &longnull, output->Geo.LOSE.sunazimuth,  &anynull, &status);
		*status_ = *status_ + status;
		//fits_read_col(fptr, TFLOAT, 15, 1, 1, naxis2, &longnull, output->Geo.LOSE.distlosplanetcenter,
		//			  &anynull, &status);
		//fits_read_col(fptr, TFLOAT, 16, 1, 1, naxis2, &longnull, output->Geo.LOSE.distscplanetcenter,
		//			  &anynull, &status);
	}
	else
		status = 0;
	
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_LOSE -----------------------------------------------------

// ----- read_Geo_IRFOV --------------------------------------------------------
void 
read_Geo_IRFOV(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;

        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_IRFOV",0,&status);
	
	// ------ Array Allocation ------
	output->Geo.IRFOV.sza = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.disttoplanetnp = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.pixelsize = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.anglelossun = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.phaselossun = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.solarincidence = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.solarlocaltime = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.anglenormalobserver = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.sunazimuth = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.distlosplanetcenter = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.distscplanetcenter = malloc(naxis2*sizeof(float));
	output->Geo.IRFOV.lat = calloc(naxis2,sizeof(float));
	output->Geo.IRFOV.lon = calloc(naxis2,sizeof(float));
	output->Geo.IRFOV.alt = calloc(naxis2,sizeof(float));
	output->Geo.IRFOV.ra = calloc(naxis2,sizeof(float));
	output->Geo.IRFOV.dec = calloc(naxis2,sizeof(float));
	
        // ------ Read Table data ------
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.IRFOV.lat, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.IRFOV.lon, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.IRFOV.alt, &anynull, &status);	
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.IRFOV.ra, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.IRFOV.dec, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Geo.IRFOV.sza, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 7, 1, 1, naxis2, &longnull, output->Geo.IRFOV.disttoplanetnp, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 8, 1, 1, naxis2, &longnull, output->Geo.IRFOV.pixelsize, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 9, 1, 1, naxis2, &longnull, output->Geo.IRFOV.anglelossun, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 10, 1, 1, naxis2, &longnull, output->Geo.IRFOV.phaselossun, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 11, 1, 1, naxis2, &longnull, output->Geo.IRFOV.solarincidence, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 12, 1, 1, naxis2, &longnull, output->Geo.IRFOV.solarlocaltime, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 13, 1, 1, naxis2, &longnull, output->Geo.IRFOV.anglenormalobserver,  &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 14, 1, 1, naxis2, &longnull, output->Geo.IRFOV.sunazimuth, &anynull, &status);	
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 15, 1, 1, naxis2, &longnull, output->Geo.IRFOV.distlosplanetcenter, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 16, 1, 1, naxis2, &longnull, output->Geo.IRFOV.distscplanetcenter, &anynull, &status);
	*status_ = *status_ + status;
	
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_IRFOV ----------------------------------------------------

// ----- read_Geo_Coordinates --------------------------------------------------
void 
read_Geo_Coordinates(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;

        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Coordinates",0,&status);
	
	// ------ Array Allocation ------
	output->Geo.Coordinates.xsc_x = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.xsc_y = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.xsc_z = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.ysc_x = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.ysc_y = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.ysc_z = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.zsc_x = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.zsc_y = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.zsc_z = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.angle_xsc_localvert = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.angle_xsc_proj = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p1_u = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p1_v = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p2_u = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p2_v = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p3_u = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p3_v = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p4_u = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p4_v = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p5_u = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p5_v = malloc(naxis2*sizeof(float));
	
        // ------ Read Table data ------
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Coordinates.xsc_x, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Coordinates.xsc_y, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Coordinates.xsc_z, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Coordinates.ysc_x, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Coordinates.ysc_y, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Geo.Coordinates.ysc_z, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 7, 1, 1, naxis2, &longnull, output->Geo.Coordinates.zsc_x, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 8, 1, 1, naxis2, &longnull, output->Geo.Coordinates.zsc_y, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 9, 1, 1, naxis2, &longnull, output->Geo.Coordinates.zsc_z, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 10, 1, 1, naxis2, &longnull, output->Geo.Coordinates.angle_xsc_localvert, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 11, 1, 1, naxis2, &longnull, output->Geo.Coordinates.angle_xsc_proj, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 12, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p1_u, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 13, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p1_v, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 14, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p2_u, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 15, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p2_v, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 16, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p3_u, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 17, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p3_v, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 18, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p4_u, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 19, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p4_v, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 20, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p5_u, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 21, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p5_v, &anynull, &status);
	*status_ = *status_ + status;

	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_Coordinates ----------------------------------------------

// ----- read_Geo_TransMatrix --------------------------------------------------
void 
read_Geo_TransMatrix(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;

        // ------ Move to the HDU ------
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_TransMatrix",0,&status);
	
	// ------ Array Allocation ------
	output->Geo.TransMatrix.x_ra  = malloc(naxis2*sizeof(float));
	output->Geo.TransMatrix.x_dec = malloc(naxis2*sizeof(float));
	output->Geo.TransMatrix.y_ra  = malloc(naxis2*sizeof(float));
	output->Geo.TransMatrix.y_dec = malloc(naxis2*sizeof(float));
	output->Geo.TransMatrix.z_ra  = malloc(naxis2*sizeof(float));
	output->Geo.TransMatrix.z_dec = malloc(naxis2*sizeof(float));
	
        // ------ Read Table data ------
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.x_ra, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.x_dec, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.y_ra, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.y_dec, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.z_ra, &anynull, &status);
	*status_ = *status_ + status;
	fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.z_dec, &anynull, &status);
	*status_ = *status_ + status;
	
	fits_report_error(stderr, *status_);
}
// ----- END read_Geo_TransMatrix ----------------------------------------------

// ----- read_Geo --------------------------------------------------------------
void 
read_Geo(fitsfile *fptr, spica_1Afile *output, int *status_, int mode, int uvir){
	
	int ii = 0;
	int status = 0;
	int anynull;
	long naxis2,longnull;
	int status_all = 0;
	
        // ------ Build GEO structure ------
	
        // --- UV/IR ---
        //Lecture Geo_record
	read_Geo_Record(fptr,output,status_);
	if(status == 0){
		if(mode != 0)
			printf("Geo record successfully read\n");
	}
	else
		printf("Error while reading Geo record\n");	
	status_all = status_all + *status_;
	
	//Lecture Geo_Spacecraft
	read_Geo_Spacecraft(fptr,output,status_,uvir);
	if(status == 0){
		if(mode != 0)
			printf("Geo spacecraft successfully read\n");
	}
	else
		printf("Error while reading Geo record\n");	
	status_all = status_all + *status_;
	
	//Lecture Geo_Coordinates
	read_Geo_Coordinates(fptr,output,status_);
	if(status == 0){
		if(mode != 0)
			printf("Geo Coordinates successfully read\n");
	}
	else
		printf("Error while reading Geo Coordinates\n");	
	status_all = status_all + *status_;

	//Lecture Geo_TransMatrix
	read_Geo_TransMatrix(fptr,output,status_);
	if(status == 0){
		if(mode != 0)
			printf("Geo TransMatrix successfully read\n");
	}
	else
		printf("Error while reading Geo TransMatrix\n");	
	status_all = status_all + *status_;	
	
        // --- UV ONLY ---
	if(uvir == 0)
	{
		//Lecture Geo_Band3
		read_Geo_Band3(fptr,output,status_);
		if(status == 0){
			if(mode != 0)
				printf("Geo band3 successfully read\n");
		}
		else
			printf("Error while reading Geo band3\n");	
		status_all = status_all + *status_;
		
		//Lecture Geo_Band1,2,4 et 5 ou CCDLine
		if (output->Parameters.codeop >= 100) {
			
			read_Geo_Band1(fptr,output,status_);
			if(status == 0){
				if(mode != 0)
					printf("Geo band1 successfully read\n");
			}
			else
				printf("Error while reading Geo band1\n");	
			status_all = status_all + *status_;
			
			read_Geo_Band2(fptr,output,status_);
			if(status == 0){
				if(mode != 0)
					printf("Geo band2 successfully read\n");
			}
			else
				printf("Error while reading Geo band2\n");	
			status_all = status_all + *status_;		
			
			read_Geo_Band4(fptr,output,status_);
			if(status == 0){
				if(mode != 0)
					printf("Geo band4 successfully read\n");
			}
			else
				printf("Error while reading Geo band4\n");	
			status_all = status_all + *status_;
	
			read_Geo_Band5(fptr,output,status_);
			if(status == 0){
				if(mode != 0)
					printf("Geo band5 successfully read\n");
			}
			else
				printf("Error while reading Geo band5\n");	
			status_all = status_all + *status_;		
		}
		else {
			
			read_Geo_CCDLine(fptr,output,status_);
			if(status == 0){
				if(mode != 0)
					printf("Geo CCDLine successfully read\n");
			}
			else
				printf("Error while reading Geo CCDLine\n");	
			status_all = status_all + *status_;
		}
		
		//Lecture Geo_LOSE
		read_Geo_LOSE(fptr,output,status_);
		if(status == 0){
			if(mode != 0)
				printf("Geo LOSE successfully read\n");
		}
		else
			printf("Error while reading Geo LOSE\n");	
		status_all = status_all + *status_;
	}
	
        // --- IR ONLY ---
        if(uvir == 1)
        {
		//Lecture Geo_IRFOV
		read_Geo_IRFOV(fptr,output,status_);
		if(status == 0){
			if(mode != 0)
				printf("Geo IRFOV successfully read\n");
		}
		else
			printf("Error while reading Geo band3\n");	
		status_all = status_all + *status_;
	}

	*status_ = status_all;
}
// ----- END read_Geo ----------------------------------------------------------

// ----- read_CleanData --------------------------------------------------------
void 
read_CleanData(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii,anynull;
	int fpixel = 1;
	float nullval = 0;
	int status = 0;
	long npixelsperimg;
	
	npixelsperimg = output->Info.naxis1 * output->Info.naxis2;
	
	for (ii=0; ii<output->Info.naxis3; ii++) {
		output->cleandata[ii] = malloc(npixelsperimg*sizeof(float));
		fits_read_img(fptr, TFLOAT, fpixel+ii*npixelsperimg,npixelsperimg, &nullval,
					  output->cleandata[ii], &anynull, &status);
	}
	
	fits_report_error(stderr, *status_);
	
}
// ----- END read_CleanData ----------------------------------------------------

// ----- read_Flag -------------------------------------------------------------
void 
read_Flag(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii,anynull;
	int nullval = 0;
	int fpixel = 1;
	int status = 0;
	long npixelsperimg;
	
	fits_movnam_hdu(fptr,IMAGE_HDU,"Flag",0,&status);
		
	npixelsperimg = output->Info.naxis1 * output->Info.naxis2;
	
	for (ii=0; ii<output->Info.naxis3; ii++) {
		output->flag[ii] = malloc(npixelsperimg*sizeof(int));
		fits_read_img(fptr, TINT, fpixel+ii*npixelsperimg,npixelsperimg, &nullval,
					  output->flag[ii], &anynull, &status);
	}
	
	fits_report_error(stderr, *status_);
	
}
// ----- END read_Flag ---------------------------------------------------------

// ----- apply_Flag ------------------------------------------------------------
void 
apply_Flag(spica_1Afile *output){
	
	int ii,ij,ik;
	float NaN = sqrt(-1);
		
	for (ii=0; ii<output->Info.naxis3; ii++) {
		
		for (ij=0; ij<output->Info.naxis2; ij++) {
			
			for (ik=0; ik<output->Info.naxis1; ik++) {
				
				if ( ((output->flag[ii][ik+ij*output->Info.naxis1]) == 1)
				||   ((output->flag[ii][ik+ij*output->Info.naxis1]) == 2)
				||   ((output->flag[ii][ik+ij*output->Info.naxis1]) == 3)
				||   ((output->flag[ii][ik+ij*output->Info.naxis1]) == 4) )
					output->cleandata[ii][ik+ij*output->Info.naxis1] = NaN;
			}
		}
		
	}
	
}
// ----- END apply_Flag --------------------------------------------------------

// ----- read_ErrData ----------------------------------------------------------
void 
read_ErrData(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii,anynull;
	int fpixel = 1;
	int nullval = 0;
	int status = 0;
	long npixelsperimg;
	
	fits_movnam_hdu(fptr,IMAGE_HDU,"ErrData",0,&status);
		
	npixelsperimg = output->Info.naxis1 * output->Info.naxis2;
	
	for (ii=0; ii<output->Info.naxis3; ii++) {
		output->errdata[ii] = malloc(npixelsperimg*sizeof(int));
		fits_read_img(fptr, TFLOAT, fpixel+ii*npixelsperimg,npixelsperimg, &nullval,
					  output->errdata[ii], &anynull, &status);
	}
	
	fits_report_error(stderr, *status_);
	
}
// ----- END read_ErrData ------------------------------------------------------

// ----- read_WL ---------------------------------------------------------------
void 
read_WL(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii,anynull;
	int fpixel = 1;
	int nullval = 0;
	int status = 0;
	long npixelsperimg;
	
	fits_movnam_hdu(fptr,IMAGE_HDU,"WAVELENGTH",0,&status);
		
	npixelsperimg = output->Info.naxis1 * output->Info.naxis2;
	
	for (ii=0; ii<output->Info.naxis3; ii++) {
		output->WL[ii] = malloc(npixelsperimg*sizeof(int));
		fits_read_img(fptr, TFLOAT, fpixel+ii*npixelsperimg,npixelsperimg, &nullval,
					  output->WL[ii], &anynull, &status);
	}
	
	fits_report_error(stderr, *status_);
	
}
// ----- END read_WL -----------------------------------------------------------

// ----- read_DC ---------------------------------------------------------------
void 
read_DC(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii,anynull;
	int fpixel = 1;
	int nullval = 0;
	int status = 0;
	long npixelsperimg;
	
	fits_movnam_hdu(fptr,IMAGE_HDU,"DC",0,&status);
		
	npixelsperimg = output->Info.naxis1 * output->Info.naxis2;
	
	for (ii=0; ii<output->Info.naxis3; ii++) {
		output->DC[ii] = malloc(npixelsperimg*sizeof(int));
		fits_read_img(fptr, TFLOAT, fpixel+ii*npixelsperimg,npixelsperimg, &nullval,
					  output->DC[ii], &anynull, &status);
	}
	
	fits_report_error(stderr, *status_);
	
}
// ----- END read_DC -----------------------------------------------------------

// ----- read_RAW --------------------------------------------------------------
void 
read_RAW(fitsfile *fptr, spica_1Afile *output,int *status_){
	
	int ii,anynull;
	int fpixel = 1;
	int nullval = 0;
	int status = 0;
	long npixelsperimg;
	
	fits_movnam_hdu(fptr,IMAGE_HDU,"RAW",0,&status);
		
	npixelsperimg = output->Info.naxis1 * output->Info.naxis2;
	
	for (ii=0; ii<output->Info.naxis3; ii++) {
		output->RAW[ii] = malloc(npixelsperimg*sizeof(int));
		fits_read_img(fptr, TFLOAT, fpixel+ii*npixelsperimg,npixelsperimg, &nullval,
					  output->RAW[ii], &anynull, &status);
	}
	
	fits_report_error(stderr, *status_);
	
}
// ----- END read_RAW ----------------------------------------------------------

// ----- trimchars -------------------------------------------------------------
char 
*trimchars(char *str)
{
	char *end;
	
	// Trim begin '
	str++;
	
	// Trim leading space
	while(isspace(*str)) str++;
	
	if(*str == 0)  // All spaces?
		return str;
	
	// Trim end '
	end = str + strlen(str) - 1;
	end--;
	
	// Trim trailing space
	while(end > str && isspace(*end)) end--;
	
	// Write new null terminator
	*(end+1) = 0;
	
	return str;
}
// ----- END trimchars ---------------------------------------------------------

